@@ -9,22 +9,23 @@ from irc.buffer import DecodingLineBuffer
 
 class Pyromancer(object):
 
-    def __init__(self, settings):
-        self.parse_settings(settings)
+    def __init__(self, settings_path):
+        self.settings = Settings(settings_path)
         self.find_commands()
 
     def connect(self):
-        return Connection(self.host, self.port, self.encoding)
+        return Connection(self.settings.host, self.settings.port,
+                          self.settings.encoding)
 
     def run(self):
         self.connection = self.connect()
         self.online = True
 
-        self.connection.write('NICK {}\n'.format(self.nick))
-        self.connection.write('USER {0} {1} {1} :{2}\n'.format(self.nick,
-                                                               self.host,
-                                                               self.real_name))
+        self.connection.write('NICK {}\n'.format(self.settings.nick))
+        self.connection.write('USER {0} {1} {1} :{2}\n'.format(
+            self.settings.nick, self.settings.host, self.settings.real_name))
 
+        ticks = self.settings.ticks
         while self.online:
             self.connection.read()
 
@@ -35,37 +36,84 @@ class Pyromancer(object):
                     self.connection.write('PONG {}\n'.format(line[1]))
 
                 for c in self.commands:
-                    c.command.match(line, self.connection, self.command_prefix)
+                    c.command.match(line, self.connection, self.settings)
 
-            time.sleep(1.0 / self.ticks)
+            time.sleep(1.0 / ticks)
 
     def find_commands(self):
         self.commands = []
 
-        for m in self.packages:
-            package, submodule = m.split('.', 1)
+        for package in self.settings.packages:
+            if isinstance(package, tuple):
+                package_settings = package[1]
+                package = package[0]
+            else:
+                package_settings = {}
 
-            if not package:
-                package = 'pyromancer'
+            ignored = package_settings.get('disabled_commands', [])
+            ignored = ['{}.{}'.format(package, i) if not i.startswith(package)
+                       else i for i in ignored]
 
-            module = importlib.import_module(
-                '{}.commands.{}'.format(package, submodule))
+            module_name = '{}.commands'.format(package)
+            if module_name in ignored:
+                continue
 
-            functions = inspect.getmembers(module, inspect.isfunction)
+            module = importlib.import_module(module_name)
 
-            self.commands.extend(f for fn, f in functions
-                                 if hasattr(f, 'command'))
+            modules = [('', module,)]
+            modules.extend(inspect.getmembers(module, inspect.ismodule))
 
-    def parse_settings(self, settings):
-        self.host = settings.get('host', '')
-        self.port = settings.get('port', 6667)
-        self.encoding = settings.get('encoding', 'utf8')
-        self.nick = settings['nick']
-        self.ident = settings.get('ident', self.nick)
-        self.real_name = settings.get('real_name', self.nick)
-        self.ticks = settings.get('ticks', 10)
-        self.packages = settings.get('packages', [])
-        self.command_prefix = settings.get('prefix', '!')
+            for name, module in modules:
+                if module.__name__ in ignored:
+                    continue
+
+                functions = inspect.getmembers(module, inspect.isfunction)
+                self.commands.extend(
+                    f for fn, f in functions if
+                    hasattr(f, 'command') and '{}.{}'.format(
+                        module.__name__, fn) not in ignored)
+
+
+class Settings(object):
+
+    def __init__(self, path):
+        main_settings = importlib.import_module(path)
+        self.packages = getattr(main_settings, 'packages', [])
+        self.package_settings = {}
+        self.package_name, _ = path.split('.', 1)
+
+        if self.package_name not in self.packages:
+            self.packages.insert(0, self.package_name)
+
+        for package in self.packages:
+            if isinstance(package, tuple):
+                package = package[0]
+
+            if package == self.package_name:
+                module = main_settings
+            else:
+                module = importlib.import_module('{}.settings'.format(package))
+
+            self.package_settings[package] = module
+
+        self.global_settings = None
+        if 'pyromancer' not in self.packages:
+            self.global_settings = importlib.import_module(
+                'pyromancer.settings')
+
+    def __getattr__(self, item):
+        for package in self.packages:
+            if isinstance(package, tuple):
+                package = package[0]
+
+            if hasattr(self.package_settings[package], item):
+                return getattr(self.package_settings[package], item)
+
+        if hasattr(self.global_settings, item):
+            return getattr(self.global_settings, item)
+
+        raise AttributeError('No such setting "{}" found in any of the '
+                             'installed packages'.format(item))
 
 
 class Connection(object):
